/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.database.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.naming.InitialContextInitializer;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: DBSchemaCreator.java 5794 2006-05-26 15:23:31Z peterit $
 */

public class DBSchemaCreator  {
  
  private static Log log = ExoLogger.getLogger("database.DBSchemaCreator");
  
  private List <CreateDBSchemaPlugin> createDBSchemaPlugins = new ArrayList<CreateDBSchemaPlugin>();
  
  public DBSchemaCreator(InitialContextInitializer contextInit) {

  }
  
  // for testing only
  private DBSchemaCreator(String dsName, String script) throws SQLException, NamingException {
    createTables(dsName, script);
  }

  public void createTables(String dsName, String script) throws NamingException, SQLException {
    InitialContext context = new InitialContext();
    Object obj = context.lookup(dsName);
    
    DataSource ds;
    /// TODO
//    if(obj instanceof Reference) {
//      Reference ref = (Reference) obj;
//      String factoryCN = ref.getFactoryClassName();
//      try {
//        ObjectFactory factory = (ObjectFactory)Class.forName(factoryCN).newInstance();
//        obj = factory.getObjectInstance(ref, null, null, null);
//        //System.out.println("Lookup: "+name+" obj: "+obj+" factory: "+factory);
//      } catch (Exception e) {
//        e.printStackTrace();
//        throw new NamingException("Exception: "+e);
//      }
//      ds = (DataSource)obj;
//    } else
      
    ds = (DataSource)context.lookup(dsName);
    
    Connection conn = ds.getConnection();
    conn.setAutoCommit(false);
    try {
      String[] scripts = script.split(";");
      for (String scr: scripts) {
        log.debug("Execute script: \n"+scr);
        conn.createStatement().executeUpdate(scr);
      }
      conn.commit();
    } catch (SQLException e) {
      conn.rollback();  
      SQLException next = e.getNextException(); 
      String errorTrace = "";
      while (next != null) {
        errorTrace += next.getMessage() + "; ";
        next = e.getNextException();
      }
      Throwable cause = e.getCause(); 
      log.error("Could not create db schema for " + dsName 
          + ". Reason: " + e.getMessage() + "; " + errorTrace
          + (cause != null ? " (Cause: " + cause.getMessage() + ")" : "")); //, e
    } finally {
      conn.close();
    }
    log.info("DB schema created succesfully. "+dsName+" context "+context);
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof CreateDBSchemaPlugin) {
      CreateDBSchemaPlugin csplugin = (CreateDBSchemaPlugin) plugin;
      try {
        createTables(csplugin.getDataSource(), csplugin.getScript());
        createDBSchemaPlugins.add(csplugin);
      } catch (NamingException e) {
        e.printStackTrace();
      } catch (SQLException e) {
        e.printStackTrace();
      }
    }
      
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return createDBSchemaPlugins;
  }
  
  // for testing
  public static DBSchemaCreator initialize(String dsName, String script) throws SQLException, NamingException {
    return new DBSchemaCreator(dsName, script);
  }

}
