/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.document.impl;

import java.io.InputStream;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.document.DocumentReader;

import org.htmlparser.Parser;
import org.htmlparser.beans.StringBean;
import org.htmlparser.util.ParserException;
import org.htmlparser.visitors.NodeVisitor;

import java.util.regex.PatternSyntaxException;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



/**
 * Created by The eXo Platform SARL
 *
 * A parser of HTML files.
 * @author <a href="mailto:zagrebin_v@mail.ru">Victor Zagrebin</a>
 * @version March 04, 2006
 */
public class HTMLDocumentReader extends BaseDocumentReader {

  /**
   * Initializes a newly created object for text/html files format parsing.
   * @param params the container parameters.
   */
  public HTMLDocumentReader(InitParams params) {
  }

  /**
   * Get the text/html mime type.
   * @return The string with text/html mime type.
   */
  public String getMimeTypes() {
    return "text/html" ;
  }

  /**
   * Returns a text from html file content without user's tags and their bodies.
   * @param is an input stream with html file content.
   * @return The string only with text from file content.
   */
  public String getContentAsText(InputStream is) throws Exception {
    String text = "";
    try {
      int size = is.available();
      byte b[] = new byte[size];
      is.read(b);
      String html = new String(b);
      StringBean sb = new StringBean();
      Parser parser = Parser.createParser(html, null);
      parser.visitAllNodesWith(sb);
      text = sb.getStrings();
      sb.setLinks(true);
      parser.reset();
      parser.visitAllNodesWith(sb);
      //String s1 = sb.getStrings();
      text = new String(sb.getStrings().getBytes());
      text = this.delete(text);
    }
    catch(Exception e){
    }

    return text ;
  }


  /**
   * Cleans the string from users's tags and their bodies.
   * @param str the string which contain a text with user's tags.
   * @return The string cleaned from user's tags and their bodies.
   */
  public String delete(String str) {
    try {
      StringBuffer input = new StringBuffer(str);
      String patternString = "<+[^>]*>+";
      Pattern pattern = Pattern.compile(patternString, Pattern.CASE_INSENSITIVE+
                                                  Pattern.DOTALL);
      Matcher matcher = pattern.matcher(input);
      while(matcher.find()) {
        int start = matcher.start();
        int end = matcher.end();
        input.delete(start, end);
        matcher = pattern.matcher(input);
      }
      return input.substring(0,input.length());
    }
    catch (PatternSyntaxException e) {
    }
    return "";
  }
}
