/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.ldap;

import javax.naming.NamingEnumeration;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.HasControls;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.PagedResultsControl;
import javax.naming.ldap.PagedResultsResponseControl;
import javax.naming.ldap.SortControl;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.net.NetService;
import org.exoplatform.test.BasicTestCase;
/**
 * Created by The eXo Platform SARL
 * Author : Nhu Thuan
 *          thuannd@exoplatform.com
 * Oct 3, 2005
 */
public class TestLDAPService extends BasicTestCase {
  final  static public String ROOT_DN = "cn=Manager,dc=exoplatform,dc=org" ;
  final  static public String ROOT_PASSWORD = "secret" ;
  final  static public String DEVELOPER_UNIT_DN = "ou=developer,o=company,c=vietnam,dc=exoplatform,dc=org" ;
  final  static public String EXO_DEVELOPER_DN = "cn=exo, " + DEVELOPER_UNIT_DN ;
  static private String       LDAP_HOST = "127.0.0.1" ;
  static private int          LDAP_PORT = 389 ;
  
  private LDAPService service_;
  private NetService nservice_ ;   
  
  private boolean test = true ;
  
  public void setUp() throws Exception {
    PortalContainer pcontainer = PortalContainer.getInstance();
    nservice_ = (NetService)pcontainer.getComponentInstanceOfType(NetService.class) ;
    if(nservice_.ping(LDAP_HOST,LDAP_PORT)<0) {      
      test = false ;
      return ;
    }
    service_ = (LDAPService) pcontainer.getComponentInstanceOfType(LDAPService.class); 
        
  }
  
  public void testLDAPService() throws Exception {    
    if(!test) {
      System.out.println("===LDAP Server is not started on host:"+LDAP_HOST + " port:"+LDAP_PORT) ;
      return ; 
    }
    LdapContext ctx = service_.getLdapContext() ;
    // Create attributes to be associated with the new context
    System.out.println("##############################CREATE##############################");
    // Create the context
    for(int i = 0; i < 10; i++) {
      String cn = "exo" + i ;
      String sn = "sn" + i ;
      String dn = "cn="+ cn + ", " + DEVELOPER_UNIT_DN ;
      Attributes attrs = new BasicAttributes(true); // case-ignore
      attrs.put(new BasicAttribute( "objectClass", "person"));                
      attrs.put(new BasicAttribute("cn", cn));          
      attrs.put(new BasicAttribute("sn", sn));        
      attrs.put(new BasicAttribute("telephonenumber", "0989654990"));   
      ctx.createSubcontext(dn, attrs);
      System.out.println(ctx.getAttributes(dn));
    }
    
    // Search for objects that have those matching attributes
    System.out.println("#################SEARCH BY ATTRIBUTES#############################");
    Attributes matchAttrs = new BasicAttributes(true); // ignore attribute name case
    matchAttrs.put(new BasicAttribute("sn", "sn1"));
    matchAttrs.put(new BasicAttribute("telephonenumber", "0989654990"));
    NamingEnumeration results = ctx.search(DEVELOPER_UNIT_DN, matchAttrs);
    while (results.hasMore()) {
      SearchResult sr = (SearchResult)results.next();
      printAttributes(sr.getName(), sr.getAttributes()) ; 
    }
    System.out.println("#################SEARCH BY QUERY FILTER##########################");
    SearchControls  searchControls = new SearchControls() ;
    searchControls.setCountLimit(10) ;
    searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE)  ;
    Control[] requestControls = {new PagedResultsControl(5, Control.CRITICAL) ,
                                 new SortControl(new String[]{"cn"}, Control.NONCRITICAL)} ;
    ctx.setRequestControls(requestControls);
    results = ctx.search(DEVELOPER_UNIT_DN,"(objectclass=person)", searchControls);
    while (results.hasMoreElements()) {
      SearchResult sr = (SearchResult)results.nextElement();
      printAttributes(sr.getName(), sr.getAttributes()) ; 
      if (sr instanceof HasControls) {
        Control[] controls = ((HasControls)sr).getControls();
        if (controls != null) {
          System.out.println("====================>response control is not null");
          for (int i = 0; i < controls.length; i++) {
            if (controls[i] instanceof PagedResultsResponseControl) {
              PagedResultsResponseControl prrc =  (PagedResultsResponseControl)controls[i];
              System.out.println("page result size: " + prrc.getResultSize());
              System.out.println("cookie: " + prrc.getCookie());
            } else {
              // Handle other response controls (if any)
            }
          }
        }
      }
    }
    
    Control[] controls = ctx.getResponseControls();
    if (controls != null) {
      System.out.println("====================>response control is not null");
      for (int i = 0; i < controls.length; i++) {
        if (controls[i] instanceof PagedResultsResponseControl) {
          PagedResultsResponseControl prrc =  (PagedResultsResponseControl)controls[i];
          System.out.println("page result size: " + prrc.getResultSize());
          System.out.println("cookie: " + prrc.getCookie());
        } else {
          // Handle other response controls (if any)
        }
      }
    }
    ctx.setRequestControls(null);
    System.out.println("##############################REMOVE##############################");
    for(int i = 0; i < 10; i++) {
      String cn = "exo" + i ;
      String dn = "cn="+ cn + ", " + DEVELOPER_UNIT_DN ;
      ctx.unbind(dn) ;
      System.out.println("remove " +  dn + " successfully");
    }
  }
  
  private  void printAttributes(String entry, Attributes attrs) {
    System.out.println("entry: " + entry);
    System.out.println("    " + attrs) ; 
  }
}
