/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.templates.groovy;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 2, 2005
 * @version $Id: GroovyTemplateFileLoader.java 5799 2006-05-28 17:55:42Z geaz $
 * 
 * This class is used to load a template from one or many  different directories.
 */
public class GroovyTemplateFileLoader extends GroovyTemplateLoader {
  private String[] basePath_ ;
  
  /**
   * Create a new template loader that should load the template from a file  system in a 
   * specified directory
   * @param basePath  The location of  the  templates. The  full path  of a template should
   * be basePath + "/"  +  templateName
   */
  public GroovyTemplateFileLoader(String basePath) {
    String[] tmp = new String[] {basePath} ;
    init(tmp) ;
  }
  /**
   * Create a new template loader that should load the template from a file  system in the 
   * specified directories
   * @param basePath The locations of the templates.  The  full path of  a template should be 
   * in one  basePath[x]  + "/" + templateName
   */
  public GroovyTemplateFileLoader(String[] basePath) {
    init(basePath) ;
  }
  
  /**
   * process the paths and normalize them
   * @param basePath
   */
  private void init(String[] basePath) {
    basePath_ = new String[basePath.length] ;
    for(int i = 0; i < basePath.length ; i++) {
      if(!basePath[i].endsWith("/")) basePath_[i] = basePath[i] + "/" ;
      else basePath_[i] = basePath[i] ;
    }
  }
  /**
   * If the type is null, not specified,  or  'file' , check if a  file with the path
   * basePath_[x] + "/" + template  is existed.
   */
  public boolean canLoad(String type, String template) throws Exception {
    if(type == null ||  "file".equals(type)) {
      for(int i = 0; i < basePath_.length ; i++) {
        String realPath =  basePath_[i] + template ;
        File file = new File(realPath) ;
        return file.exists() ;
      }
    }
    return false ;
  }
  
  public InputStream load(String template) throws Exception {
    for(int i = 0; i < basePath_.length ; i++) {
      String realPath =  basePath_[i] + template ;
      File file = new File(realPath) ;
      if(file.exists()) {
        FileInputStream is = new FileInputStream(file) ;
        byte[]  buf =  new  byte[is.available()] ;
        is.read(buf) ;
        ByteArrayInputStream bis = new ByteArrayInputStream(buf) ;
        return bis;
      }
    } 
    return null ;
  }
}
