/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.util.* ;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator ;

import org.apache.commons.fileupload.FileItem;
import org.exoplatform.commons.utils.ISO8601;


public class UIDateTimeInput extends UIInput {
  public static final String  COMPONENT_FAMILY = "org.exoplatform.faces.core.component.UIDateTimeInput" ;
  public static final String DAY_EXTENSION = "_dd";
  public static final String MONTH_EXTENSION = "_mm";
  public static final String YEAR_EXTENSION = "_yyyy";  
  public static final String HOUR_EXTENSION = "_hh";
  public static final String MINUTE_EXTENSION = "_min";

  protected GregorianCalendar calendar_ ;
  private int minuteStep_;
  private boolean updateOnChangeAction_;
  private int minYearRange_ = GregorianCalendar.getInstance().get(GregorianCalendar.YEAR) - 5 ;
  private int maxYearRange_ = GregorianCalendar.getInstance().get(GregorianCalendar.YEAR) + 5 ;
  
  public UIDateTimeInput(String name, Date date) {
  	  setId(name);
    name_ = name;
    calendar_ = new GregorianCalendar();
    calendar_.setTime(date);
    minuteStep_ = 1;
    editable_ = true ;
    setRendererType("DateTimeInputRenderer");
  }    

  public UIDateTimeInput(String name, Date date, int minuteStep) {
  	this(name, date);
    minuteStep_ = minuteStep;
  }

  public UIDateTimeInput setUpdateOnChangeAction(boolean b) { 
    updateOnChangeAction_ = b ;
    return this ;
  }

  public boolean getUpdateOnChangeAction() {
  	return updateOnChangeAction_;
  }
  
  public String getFamily() {  return COMPONENT_FAMILY ; }
    
  public Object getDecodedValue() { return calendar_.getTime() ; }

  public String getValue() { return ISO8601.format(calendar_); }
  
  public Calendar getCalendar() { return calendar_ ; }
  
  public UIDateTimeInput addValidator(Validator validator) {
    addComponentValidator(validator) ;
    return this ;
  }
  
  public UIDateTimeInput addValidator(Class clazz) {
    addComponentValidator(clazz) ;
    return this ;
  }
  
  public UIDateTimeInput addTime(Date date) {
    calendar_.setTime(date);
  	if (calendar_.get(GregorianCalendar.YEAR) < minYearRange_ || 
  			calendar_.get(GregorianCalendar.YEAR) > maxYearRange_) {
  	  minYearRange_ = GregorianCalendar.getInstance().get(GregorianCalendar.YEAR) - 5 ;
  	  maxYearRange_ = GregorianCalendar.getInstance().get(GregorianCalendar.YEAR) + 5 ;
  	}
    return this ;
  }

  public int getMinYearRange() { return minYearRange_ ; }

  public void setMinYearRange(int year) {
  	if (year > maxYearRange_) return;
  	if (calendar_.get(GregorianCalendar.YEAR) < year) return;
  	minYearRange_ = year ; 
  }
  
  public int getMaxYearRange() { return maxYearRange_ ; }
  public void setMaxYearRange(int year) { 
  	if (year < minYearRange_) return;
  	if (calendar_.get(GregorianCalendar.YEAR) > year) return;
  	maxYearRange_ = year ;
  }
  
  public void setMinuteStep_(int minuteStep) {
	minuteStep_ = minuteStep;
  }

  public int getMinuteStep_() {
	return minuteStep_;
  }

  final public  void processValidators(FacesContext context) {
    processValidators(context, calendar_) ;
  }
  
  public void decodeFromMultipartFields(FacesContext context,
      UIComponent component, List items) {
    int dayValue = 0;
    int monthValue = 0;
    int yearValue = 0;
    int hourValue = 0;
    int minuteValue = 0;
    Iterator iter = items.iterator();
    while (iter.hasNext()) {
      FileItem item = (FileItem) iter.next();
      if (item.isFormField()) {
        String name = item.getFieldName();
        if(name.equals(getName() + DAY_EXTENSION))
          dayValue = Integer.parseInt(item.getString());
        else if(name.equals(getName() + MONTH_EXTENSION))
          monthValue = Integer.parseInt(item.getString());
        else if(name.equals(getName() + YEAR_EXTENSION))
          yearValue = Integer.parseInt(item.getString());
        else if(name.equals(getName() + HOUR_EXTENSION))
          hourValue = Integer.parseInt(item.getString());
        else if(name.equals(getName() + MINUTE_EXTENSION))
          minuteValue = Integer.parseInt(item.getString());
      }          
    }
    GregorianCalendar gCal = new GregorianCalendar(yearValue, monthValue, dayValue, hourValue, minuteValue, 0);
    addTime(gCal.getTime());    
  }  
}
