/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portal.faces.component;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.faces.component.model.PortletCategoryData;
import org.exoplatform.portal.faces.listener.share.ShowLastBodyComponentActionListener;
import org.exoplatform.portal.skin.SkinConfigService;
import org.exoplatform.portal.skin.model.Style;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
import org.exoplatform.services.portletregistery.Portlet;
import org.exoplatform.services.portletregistery.PortletCategory;
import org.exoplatform.services.portletregistery.PortletPermission;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created by Tuan Nguyen
 * Date: 11 macr 2005
 */
public class UIPortletSelector extends UIExoCommand {
  
  public static final String SELECT_CATEGORY = "selecCategory";
  public static final String SELECT_PORTLET = "selecPortlet";
  public static final String REFRESH = "refresh";
  
  private static Parameter[] SELECT_CATEGORY_PARAMS = {new Parameter(ACTION , SELECT_CATEGORY)};
  private static Parameter[] REFRESH_PARAMS = {new Parameter(ACTION , REFRESH)};
  private static Parameter[] SELECT_PORTLET_PARAMS = {new Parameter(ACTION , SELECT_PORTLET)};
  private static Parameter[] CANCEL_PARAMS = {new Parameter(ACTION , CANCEL_ACTION)};
  
  Map categories_ = new LinkedHashMap() ;
  UIContainer uiContainer_ ;

  public UIPortletSelector() throws Exception {
    setRendererType("VelocityRenderer");
    setId("UIPortletSelecttor");
    addActionListener(SelectCategoryActionListener.class, SELECT_CATEGORY);
    addActionListener(SelectPortletActionListener.class, SELECT_PORTLET);
    addActionListener(ShowLastBodyComponentActionListener.class, CANCEL_ACTION);
    addActionListener(RefreshActionListener.class, REFRESH);    
    refresh() ;
  }

  public void refresh() throws Exception {
  	categories_.clear() ;
    OrganizationService orgService =
       (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
    ExternalContext econtext = FacesContext.getCurrentInstance().getExternalContext() ;
    Collection memberships = orgService.getMembershipHandler().findMembershipsByUser(econtext.getRemoteUser());
    Iterator mitr = memberships.iterator() ;
    Map<String, String> userMemberships = new HashMap<String, String>() ;
    while(mitr.hasNext()) {
      Membership m = (Membership) mitr.next() ;
      String pexpression = m.getMembershipType() + ":" + m.getGroupId() ;
      userMemberships.put(pexpression, pexpression) ;
      pexpression = "*:" + m.getGroupId() ;
      userMemberships.put(pexpression, pexpression) ;
    }
    PortletRegisteryService service =
      (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
    List list = service.getPortletCategories() ;
    Collections.sort(list, new PortletCategoryComparator()) ;
    PortletComparator portletComparator = new PortletComparator() ;
    for(int i =0; i < list.size(); i++) {
    	PortletCategory portletCategory = (PortletCategory) list.get(i);
    	List portlets = service.getPortlets(portletCategory.getId()) ;
      Iterator portletItr = portlets.iterator() ;
      while(portletItr.hasNext()) {
        Portlet portlet = (Portlet) portletItr.next() ;
        Collection temp = service.getPortletPermissions(portlet.getId());
        Iterator permissionItr = temp.iterator() ;
        boolean hasRole = false ;
        while(permissionItr.hasNext() && !hasRole) {
          PortletPermission role = (PortletPermission) permissionItr.next() ;
          hasRole = userMemberships.containsKey(role.getPermissionExpression())  ;
        }
        if(!hasRole) portletItr.remove() ;
      }
      if(portlets.size() > 0) {
        Collections.sort(portlets, portletComparator) ;
        PortletCategoryData data = new PortletCategoryData(portletCategory, portlets) ;
        categories_.put(portletCategory.getId(), data) ;
      }
    }
    if(categories_.size() > 0) {
    	PortletCategoryData select = 
    	 	(PortletCategoryData)categories_.values().iterator().next() ;
    	select.setSelect(true) ;
    }
  }
  
  public void setUIContainer(UIContainer container) {  uiContainer_ = container ; }
  
  public Parameter[]  getSelectCategoryParams() { return SELECT_CATEGORY_PARAMS ; }
  public Parameter[]  getSelectPortletParams() { return SELECT_PORTLET_PARAMS ; }
  public Parameter[]  getCancelParams() { return  CANCEL_PARAMS ; }
  public Parameter[]  getRefreshParams() { return REFRESH_PARAMS ; }
  
  public Map getPortletCaterories() { return categories_; }
  
  public boolean hasCategory() {
    if(categories_.isEmpty())  return false;
    return true;
  }
  
  private Portlet findPortlet(String id) {
    Iterator catIterator = categories_.values().iterator() ;
    while(catIterator.hasNext()) {
      PortletCategoryData cat = (PortletCategoryData) catIterator.next() ;
      Iterator portletIterator = cat.getPortlets().iterator() ;
      while(portletIterator.hasNext()) {
        Portlet portlet = (Portlet) portletIterator.next() ;
        if(id.equals(portlet.getId())) return portlet ;
      }
    }
    return null ;
  }
  
  static public class RefreshActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletSelector uiSelector = (UIPortletSelector) event.getComponent() ;
      uiSelector.refresh() ;
    }
  }
  
  static public class SelectCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	UIPortletSelector uiSelector = (UIPortletSelector) event.getComponent() ;
    	String categoryId = event.getParameter(OBJECTID) ;
    	PortletCategoryData portletCategoryData = 
    		(PortletCategoryData) uiSelector.categories_.get(categoryId) ;
    	portletCategoryData.setSelect(!portletCategoryData.isSelect()) ;
    }
  }
  
  static public class SelectPortletActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletSelector uiSelector = (UIPortletSelector) event.getComponent() ;
      String objectid = event.getParameter(OBJECTID) ;
      Portlet portlet = uiSelector.findPortlet(objectid) ;
      
      UIPortal uiPortal = (UIPortal) uiSelector.getAncestorOfType(UIPortal.class) ;
      String appName = portlet.getPortletApplicationName() ;
      String portletName = portlet.getPortletName() ;
      String portletId = appName + "/" + portletName ;
      SkinConfigService skinService =
        (SkinConfigService) PortalContainer.getComponent(SkinConfigService.class) ;
      List styles = skinService.getPortletStyles(portletId) ;
      if(styles == null) {
        styles = skinService.getPortletStyles("default") ;
      }
      String styleName = ((Style) styles.get(0)).getName() ;
      PortalComponentIDGenerator idGenerator =
        (PortalComponentIDGenerator) PortalContainer.getComponent(PortalComponentIDGenerator.class) ;
       String id = idGenerator.generatePortletId(uiPortal, portletName) ;
       UIPortlet uiPortlet = new UIPortlet(uiPortal.getOwner(), appName, portletName, id);
       uiPortlet.setMode(UIPortalComponent.COMPONENT_EDIT_MODE) ;
       org.exoplatform.portal.config.model.Portlet model = uiPortlet.getEditablePortletModel() ;
       model.setRenderer("default") ;
       model.setPortletStyle(styleName) ;
       model.setDecorator("default") ;
       uiSelector.uiContainer_.getChildren().add(uiPortlet) ;
       uiSelector.uiContainer_.setComponentModified(true) ;
       UIPage uiPage = (UIPage)uiPortlet.getAncestorOfType(UIPage.class) ;
       if(uiPage != null) {
         ExoWindowID windowId = uiPortlet.getWindowId() ;
         windowId.setConfigurationSource(uiPage.getPageModel().getPageId()) ;
       }
       uiPortal.setLastBodyComponent() ;
    }
  }  
  
  static class PortletCategoryComparator implements Comparator {
    public int compare(Object arg0, Object arg1) {
      PortletCategory cat1 = (PortletCategory) arg0 ;
      PortletCategory cat2 = (PortletCategory) arg1 ;
      return cat1.getPortletCategoryName().compareTo(cat2.getPortletCategoryName()) ;
    }
  }
  
  static class PortletComparator implements Comparator {
    public int compare(Object arg0, Object arg1) {
      Portlet p1 = (Portlet) arg0 ;
      Portlet p2 = (Portlet) arg1 ;
      return p1.getPortletName().compareTo(p2.getPortletName()) ;
    }
  }
}
