/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.filter;

import java.io.IOException;
import java.net.SocketException;
import java.util.List;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.component.ComponentRequestLifecycle;
import org.exoplatform.portal.log.UserActivitiesLogService;
/**
 * Fri, May 30, 2003 @
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PublicRequestFilter.java,v 1.28 2004/11/03 01:19:46 tuan08 Exp $
 */
public class ExoRequestFilter implements Filter  {
  final public static String ADMIN_RETURN_ACTION = "admin:returnAction" ;
  final public static String ADMIN = "admin" ;
  final public static String RETURN = "return" ;
  
  PortalContainer pcontainer_ ;
  RequestFilterRule  rule_ ;
  UserActivitiesLogService uservice_ ;
  
  public void init(FilterConfig filterConfig) {
    String portalName = filterConfig.getServletContext().getServletContextName() ;
    pcontainer_ = RootContainer.getInstance().getPortalContainer(portalName) ;
    rule_ = 
      (RequestFilterRule)  pcontainer_.getComponentInstanceOfType(RequestFilterRule.class) ;
    uservice_ = 
      (UserActivitiesLogService) pcontainer_.getComponentInstanceOfType(UserActivitiesLogService.class) ;
  }
  
  public void doFilter(ServletRequest request, ServletResponse response, 
                       FilterChain chain) throws IOException, ServletException {
    PortalContainer.setInstance(pcontainer_) ;
    List<ComponentRequestLifecycle> components = 
      pcontainer_.getComponentInstancesOfType(ComponentRequestLifecycle.class) ;
    for(ComponentRequestLifecycle component : components) {
      component.startRequest(pcontainer_);
    }
    HttpServletRequest httpRequest = (HttpServletRequest) request ;
    HttpServletResponse httpResponse = (HttpServletResponse) response ;
    RequestFact fact = null ;
    try {
      fact  = new RequestFact(pcontainer_, httpRequest, httpResponse) ;
      rule_.verify(fact) ;
      if(fact.getErrorMessage() != null) {
//        System .err.println("error: " + fact.getErrorMessage())  ;
        pcontainer_.removeSessionContainer(fact.getSessionContainer().getSessionId()) ;
        httpResponse.sendRedirect(httpRequest.getContextPath() + "/access-error.jsp") ;
        return ;
      }
      if(!fact.getProcess())  return ;
      SessionContainer scontainer =  fact.getSessionContainer() ;
      synchronized(scontainer) {
        SessionContainer.setInstance(scontainer) ;
        httpRequest.setAttribute("javax.servlet.include.path_info", fact.getRequestInfo().getViewId()) ;
        chain.doFilter(request, response) ;
      }
    } catch (Throwable ex) {
      Throwable  t = ex.getCause() ;
      if(t instanceof  SocketException) {
//        System .err.println("This  exception usually cause  by the network problem or  the client abort the request") ;
      } else {
        ex.printStackTrace() ;
      }
    } finally { 
      try{
        if(fact != null) {
          long currentTime = System.currentTimeMillis() ;
          if(fact.isNewRequest() ||  fact.isChangeLink() || fact.isChangePortal()) {
            String portalOwner = fact.getSessionContainer().getOwner() ;
            String sessionId = fact.getSessionContainer().getUniqueId() ;
            String remoteUser = httpRequest.getRemoteUser() ; 
            if(remoteUser == null)  remoteUser = "guest" ;
            String accessIp = httpRequest.getRemoteAddr() ;
            uservice_.logUserActivites(sessionId,remoteUser, portalOwner, currentTime, accessIp) ;
          } else  {
            String sessionId = fact.getSessionContainer().getUniqueId() ;
            uservice_.updateLogUserActivities(sessionId, currentTime) ;
          }
        }
      } catch(Exception ex) {
        ex.printStackTrace() ;
      }
      for(ComponentRequestLifecycle component : components) {
        component.endRequest(pcontainer_);
      }
      SessionContainer.setInstance(null) ;
      PortalContainer.setInstance(null) ;                  
    }
  }
 
  public void destroy() { }
}