/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.monitor.component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.log.PortalActivitiesLogService;
import org.exoplatform.portal.log.Query;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 22 juin 2004
 */
public class UIStatisticsPortalActionLog  extends UIExoCommand  {
  PageListDataHandler dataHandler_ ;
  private UIPageListIterator uiPageIterator_ ;
  private String selectType_ = "portal";
  private Date fromDateTime_ ;
  private Date toDateTime_ ;
  private String componentId_ = "";
  private String mostAccess_ = ""; 
  private long hits_ = 0 ;
  List chartData_ ;
  final static public String VIEW_ACTION = "view" ;
  private static Parameter[] VIEW_PARAMS = {new Parameter(ACTION , VIEW_ACTION) } ;
  
  public UIStatisticsPortalActionLog() throws Exception {
    setId("UIStatisticsPortalActionLog")  ;
    setRendererType("VelocityRenderer") ;
    dataHandler_ = new PageListDataHandler() ;
    GregorianCalendar calendar = new GregorianCalendar() ;    
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;    
    fromDateTime_ = calendar.getTime() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    toDateTime_ = calendar.getTime() ; 
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_);
    getChildren().add(new UIPortalActionFilter());
    addActionListener(ViewChartActionListener.class, VIEW_ACTION) ;
    refresh() ;
  }
 
  public List getCurrentPageData() throws Exception {
    //uiPageIterator_.getPageListDataHandler().setPageList(queryLogData(selectType_)) ;
    return uiPageIterator_.getPageListDataHandler().getDatas() ;  
  }
  //public GregorianCalendar getCalendar(){ return calendar_ ; }
  public String getSelectType() { return selectType_ ; }
  public String getMostAccess() { return mostAccess_ ; }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  
  private void refresh() throws Exception{
    uiPageIterator_.getPageListDataHandler().setPageList(queryLogData(selectType_)) ;
  }
  private PageList queryLogData(String type) throws Exception {
    PortalActivitiesLogService service = 
      (PortalActivitiesLogService) PortalContainer.getComponent(PortalActivitiesLogService.class) ;
    Query q = new Query() ;
    q.setFromTimeSlot(fromDateTime_) ; 
    q.setToTimeSlot(toDateTime_) ;
    q.setComponentId(componentId_) ;
    q.setCounter(hits_) ;
    q.setOrder("Desc") ;
    PageList pageList ;
    if(type.equals("portal")){
      pageList = service.getPortalLogDatas(q) ;      
    }else if (type.equals("page")){
      pageList = service.getPageLogDatas(q) ;
    }else{
      pageList = service.getPortletLogDatas(q) ;
    }
    return pageList ;    
  }
  
  public List getChartData(String type) throws Exception {
    PortalActivitiesLogService service = 
      (PortalActivitiesLogService) PortalContainer.getComponent(PortalActivitiesLogService.class) ;
    Query q = new Query() ;
    q.setFromTimeSlot(fromDateTime_) ;
    q.setToTimeSlot(toDateTime_) ;
    q.setComponentId(componentId_) ;
    q.setCounter(hits_) ;
    q.setMostAccess(mostAccess_) ;
    if (type.equals("portal")){
      chartData_ = service.getPortalLogChartDatas(q) ;
    }else if (type.equals("page")) {
      chartData_ = service.getPageLogChartDatas(q) ;
    }else {
      chartData_ = service.getPortletLogChartDatas(q) ;
    }
    
    return chartData_ ;     
  }
  
  public void update(String type,String componentId, Date fromDate, Date toDate, long hits, 
                     String mostAcc) throws Exception {
    selectType_   = type ;
    fromDateTime_ = fromDate ;
    toDateTime_   = toDate ;
    componentId_  = componentId ;
    hits_         = hits ;
    mostAccess_   = mostAcc ;
    refresh() ;
  }  
  static public class ViewActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIStatisticsPortalActionLog uiComp  = (UIStatisticsPortalActionLog) event.getSource() ;
    }
  }
  static public class ViewChartActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIStatisticsPortalActionLog uiStatistics = (UIStatisticsPortalActionLog) event.getComponent() ;
      UIPortalActivitiesLogChart uiChart = (UIPortalActivitiesLogChart)uiStatistics.getSibling(UIPortalActivitiesLogChart.class) ;
      uiStatistics.setRenderedSibling(UIPortalActivitiesLogChart.class) ;
      List logList = uiStatistics.getChartData(uiStatistics.getSelectType());
      uiChart.updatePortalLogDataSet(logList, uiStatistics.getSelectType(), uiStatistics.getMostAccess()) ;
    }
  }
}