/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.security.component;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.faces.core.component.UIDateTimeInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.monitor.component.model.TicketData;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.ticket.impl.TicketImpl;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 23, 2006
 */
public class UITicketFilter extends UISimpleForm {
  
  private UIStringInput creator_,ticketId_ ;
  private UIDateTimeInput statDateTime_,expireDateTime_;
  private GregorianCalendar calendar;
  
  private static String REFRESH = "refresh" ;
  final public static Parameter[] REFRESH_PARAMS = {new Parameter(ACTION , REFRESH) } ;
  public UITicketFilter() {
    super("UITicketFilter", "post", null);
    setId("UITicketFilter");
    setRendererType("SimpleFormVelocityRenderer") ;
    //---------------------------time------------------------------------
    calendar = new GregorianCalendar() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    
    creator_=new UIStringInput("creator","");
    
    statDateTime_=new UIDateTimeInput("statDateTime",calendar.getTime());
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 7*24) ;
    expireDateTime_=new UIDateTimeInput("expireDateTime",calendar.getTime());    
    ticketId_=new UIStringInput("ticketId","");    
    
    add(creator_);
    add(expireDateTime_);
    add(statDateTime_);
    add(ticketId_);
    
    List<SelectItem> Status = new ArrayList<SelectItem>() ;
    Status.add(new SelectItem("          ------------          ","default")) ;
    Status.add(new SelectItem("UNLIMITED ACCESS","unlimited")) ;
    Status.add(new SelectItem( "VALID STATUS","valid")) ;
    Status.add(new SelectItem("EXPIRED STATUS","exprired")) ;
    Status.add(new SelectItem( "LOCKED STATUS","locked")) ;
    add(new UISelectBox("status", "", Status)) ;
    
    addActionListener(RefreshActionListener.class,REFRESH );
    
  }
  
  public String getRefreshParams(){ return REFRESH ; }
  
  static public class RefreshActionListener extends ExoActionListener{
    UITicketFilter uiForm ;
    public void execute(ExoActionEvent event) throws Exception {
      
      uiForm = (UITicketFilter)event.getComponent() ;
      UITicketMonitor monitor = (UITicketMonitor)uiForm.getParent() ;
      TicketData data = new TicketData() ;
      
      String creator = uiForm.getUIStringInput("creator").getValue();
      long createTime = uiForm.getUIDateTimeInput("statDateTime").getCalendar().getTimeInMillis();
      long expireTime = uiForm.getUIDateTimeInput("expireDateTime").getCalendar().getTimeInMillis();
      String ticketId = uiForm.getUIStringInput("ticketId").getValue();
      String status = uiForm.getUISelectBox("status").getValue();
      
      
      
      data.setCreator_(creator);
      data.setCreateTime_(createTime);
      data.setExpireTime_(expireTime);
      data.setTicketId_(ticketId);
      ObjectQuery oq = new ObjectQuery(TicketImpl.class);
      
      
      oq.addGT("statTime",data.getCreateTime_());
      oq.addLT("statTime",data.getExpireTime_());
      if(data.getCreator_()!=null){ oq.addEQ("creator",data.getCreator_());  }
      if(data.getTicketId_()!=null){ oq.addEQ("id",data.getTicketId_());     }
      
      if(status.equals("unlimited")) { oq.addEQ("status", -1) ; }
      if(status.equals("valid"))     { oq.addEQ("status", 0) ; }
      if(status.equals("exprired"))   { oq.addEQ("status", 1) ; }
      if(status.equals("locked"))    { oq.addEQ("status", 2) ; }
      if(status.equals("default"))   oq = null;
      monitor.update(oq);
    }
    
    public void Reset(){
      uiForm.ticketId_.setText("");      
      uiForm.creator_.setText("");      
      uiForm.statDateTime_.addTime(uiForm.calendar.getTime());
      uiForm.expireDateTime_.addTime(uiForm.calendar.getTime());   
    }
  }
}