/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.validator.ValidUserValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.community.CommunityConfigService;
import org.exoplatform.portal.community.CommunityPortal;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIGroupForm.java,v 1.11 2004/10/21 15:25:17 tuan08 Exp $
 */
public class UICommunityPortalForm extends UISimpleForm {
  static String MEMBERSHIP = "membership", PORTAL = "portal" , PRIORITY = "priority",
                DESCRIPTION = "description", HOME_PAGE = "homePage" ;
                
  static public List PRIORITIES ;
  static {
    PRIORITIES = new ArrayList() ;
    for(int i = 1; i <= 10; i++) {
      String num = Integer.toString(i) ;
      PRIORITIES.add(new SelectItem(num, num)) ;
    }
  }
  
  private CommunityPortal communityPortal_ ;
  
  public UICommunityPortalForm(OrganizationService orgService) throws Exception {
    super("communityPortalForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;
    Collection memberships = orgService.getMembershipTypeHandler().findMembershipTypes() ;
    Iterator i = memberships.iterator() ;
    List mtypes = new ArrayList(10) ;
    while(i.hasNext()) {
      MembershipType mt = (MembershipType) i.next();
      String name = mt.getName() ;
      mtypes.add(new SelectItem(name, name)) ;
    }
    add(new UISelectBox(MEMBERSHIP, "", mtypes));    
    add(new UIStringInput(PORTAL, "").addValidator(ValidUserValidator.class)) ;
    add(new UICheckBox(HOME_PAGE, "true")) ;
    add(new UISelectBox(PRIORITY, "1", PRIORITIES)) ;    
    add(new UITextArea(DESCRIPTION, "")) ;
      
    addActionListener(SaveUpdateListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
 
  public void setCommunityPortal(CommunityPortal cp) {
    communityPortal_ = cp;
    if(cp == null)  {
      getUISelectBox(MEMBERSHIP).setValue("") ;
      getUIStringInput(PORTAL).setValue("") ;
      getUISelectBox(PRIORITY).setValue("5") ;
      getUIStringInput(DESCRIPTION).setValue("") ;
      getUICheckBox(HOME_PAGE).setSelect(false) ;
    } else {
      getUISelectBox(MEMBERSHIP).setValue(cp.getMembership()) ;
      getUIStringInput(PORTAL).setValue(cp.getPortal()) ;
      getUISelectBox(PRIORITY).setValue(Integer.toString(cp.getPriority())) ;
      getUIStringInput(DESCRIPTION).setValue(cp.getDescription()) ;
      getUICheckBox(HOME_PAGE).setSelect(cp.getUseDefaultHomePage()) ;
    }
  }
  
  static public class SaveUpdateListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UICommunityPortalForm uiForm = (UICommunityPortalForm) event.getComponent() ;
      CommunityConfigService service = 
        (CommunityConfigService) PortalContainer.getComponent(CommunityConfigService.class) ;
      CommunityPortal cp = uiForm.communityPortal_ ;
      if(cp == null) {
        UIGroupExplorer uiExplorer = 
          (UIGroupExplorer) uiForm.getAncestorOfType(UIGroupExplorer.class) ;
        Group currentGroup = uiExplorer.getCurrentGroup() ;
        if(currentGroup == null) {
          Information iprovider = findInformationProvider(uiForm);
          ResourceBundle res = getApplicationResourceBundle() ;
          iprovider.addMessage(new ExoFacesMessage(res.getString("UICommunityPortalForm.msg.no-group"))) ;
          uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
          return ;
        } 
        cp = service.createCommunityPortal() ;
        cp.setGroupId(currentGroup.getId()) ;
      }
      cp.setMembership(uiForm.getUISelectBox(MEMBERSHIP).getValue()) ;
      cp.setPortal(uiForm.getUIStringInput(PORTAL).getValue()) ;
      cp.setPriority(Integer.parseInt(uiForm.getUISelectBox(PRIORITY).getValue())) ;
      cp.setDescription(uiForm.getUIStringInput(DESCRIPTION).getValue()) ;
      cp.setUseDefaultHomePage(uiForm.getUICheckBox(HOME_PAGE).getSelect()) ;
      service.addCommunityPortal(cp) ;
      UIGroupCommunityInfo uiInfo = 
        (UIGroupCommunityInfo) uiForm.getSibling(UIGroupCommunityInfo.class);
      uiInfo.setCommunityPortal(cp) ;
      uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
      UICommunityPortalForm uiForm = (UICommunityPortalForm) event.getComponent() ;
      uiForm.setRenderedSibling(UIGroupCommunityInfo.class) ;
    }
  }
}