/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.Query;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Thang Nguyen
 * @email:   thang.nguyen@exoplatform.com
 * @version: $Id: UIListUser.java,v 1.18 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UISelectListUser extends UIExoCommand {
	static Parameter[] SELECT_USER = { new Parameter(ACTION , "selectUser") } ;
  static Parameter[] REFRESH = { new Parameter(ACTION , "refresh") } ;
  static Parameter[] CANCEL = { new Parameter(ACTION , "cancel") } ;	
  
	private UIPageListIterator uiPageIterator_ ;
	private boolean adminRole_ ;
	private OrganizationService service_ ;
  private Query lastQuery_ ;
	
	public UISelectListUser(OrganizationService service) throws Exception {
		setId("UISelectListUser") ;
    setRendererType("VelocityRenderer") ;
		service_ = service  ;
		uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ; 
    adminRole_ = hasRole("admin") ;
		getChildren().add(uiPageIterator_) ;
		addActionListener(SelectUserActionListener.class, "selectUser") ;
    addActionListener(RefreshListUserActionListener.class, "refresh") ;
    addActionListener(CancelActionListener.class, "cancel") ;
		search(new Query()) ;    
	}
  
  public Parameter[] getSelectUser() { return SELECT_USER ; };
  public Parameter[] getRefreshParams() { return REFRESH ; };
  public Parameter[] getCancelParams() { return CANCEL ; };
   
  public Collection getUserList() throws Exception {
    return uiPageIterator_.getPageListDataHandler().getDatas();    
  }
   
	public void search(Query query) throws Exception {
    lastQuery_ = query ;
		uiPageIterator_.setPageList(service_.getUserHandler().findUsers(query)) ;
	}
  
  public void refresh() throws Exception {
    uiPageIterator_.setPageList(service_.getUserHandler().findUsers(lastQuery_)) ;
  }
	
	public boolean hasAdminRole() { return adminRole_ ; }

  static public class SelectUserActionListener extends ExoActionListener  {
    public SelectUserActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
		public void execute(ExoActionEvent event) throws Exception {
      UISelectListUser uiSelectListUser = (UISelectListUser) event.getComponent() ;
			String userName = event.getParameter(OBJECTID) ;
      UISelectUserManager uiSelectUserManager = (UISelectUserManager)uiSelectListUser.getParent() ; 
      
      UIMembershipForm uiMembershipForm = 
				(UIMembershipForm)uiSelectUserManager.getSibling(UIMembershipForm.class) ;
      uiMembershipForm.getUIStringInput("username").setValue(userName) ;
      UIViewUserInGroup uiViewUserInGroup = (UIViewUserInGroup) uiMembershipForm.getParent();
      uiViewUserInGroup.setShowMembership(true);
      uiSelectUserManager.setRenderedSibling(UIMembershipForm.class) ;
		}
	}
	
  static public class RefreshListUserActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectListUser uiSelectListUser = (UISelectListUser) event.getComponent() ;
      uiSelectListUser.refresh() ;
    }
  }
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectListUser uiSelectListUser = (UISelectListUser) event.getComponent() ;
      UISelectUserManager uiSelectUserManager = (UISelectUserManager)uiSelectListUser.getParent() ;
      UIViewUserInGroup uiViewUserInGroup = (UIViewUserInGroup) uiSelectUserManager.getParent();
      uiViewUserInGroup.setShowMembership(true);
      uiSelectUserManager.setRenderedSibling(UIMembershipForm.class) ;    }
  }

}