/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.user;

import condition.* ;
import suites.share.*;
import suites.share.UserLoginSuite;
import suites.share.NewAccountSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: OrganizationPortletSuite.java,v 1.3 2004/10/22 14:23:55 tuan08 Exp $
 **/
public class MembershipManagementSuite extends WebUnitSuite {
  static String DESC = "Go to Membership Management page and test all the basic actions at the membership tab" ;
  static String NAME = "MembershipManagementSuite" ;
  final static public String EXOPLATFORM_ACCOUNT = "exoaccount" ;
  
  public MembershipManagementSuite() {    
    super(NAME, DESC) ;
    init("/Site Map/Management", "/Site Map/Add User") ;   
  }  
  public MembershipManagementSuite(String path, String reg_path) {
    super(NAME, DESC) ;
    init(path, reg_path) ;
  }  
  public void  init(path, reg_path) {  
    goToPage(path) ;           
    addWebUnit(
      new ClickURLLinkUnit("GoToUsersTab", "Click on the users tab").
      setURL("op=selectNode&objectId=usernode").
      addValidator(new ExpectTextValidator("Email"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearcUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("term", EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;
    addWebUnit(
       new ClickURLLinkUnit("DeleteIfExistedAccount", "Delete the test existed account").
       setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
       addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;
    goToPage(reg_path) ;
    //create exoplatform account
    addWebSuite(new NewAccountSuite(EXOPLATFORM_ACCOUNT, EXOPLATFORM_ACCOUNT)) ;
    goToPage(path);
    //view exoplatform account
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term", EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;    
    //delete if newmembership already existed
    addWebUnit(
      new ClickURLLinkUnit("GoToMembershipsTab", "Click on the memberships tab").
      setURL("op=selectNode&objectId=UIMembershipNode").
      addValidator(new ExpectLinkWithTextValidator("Add"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteMemebershipIfExist", "remove 'newmembership' if exist").
      setURL("op=deleteMembership&objectId=newmembership").
      addCondition(new HaveLinkWithURLCondition("op=deleteMembership&objectId=newmembership"))
    ) ;
    //create new memberhip
    addWebUnit(
      new ClickLink("ClickAddMembershipType", "Click Add Membership type").
      setTextLink("Add").
      addValidator(new ExpectFormIdValidator("membershipTypeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitNewRole", "Create a new System Role").
      setFormId("membershipTypeForm").
      setField(WebUnit.ACTION, "save").
      setField("membershipName", "newmembership").
      setField("description", "This is a new membership").
      addValidator(new ExpectTextValidator("This is a new membership"))
    ) ;
    //go to user list , select an user and assign the new created membership 
    //to the user
    addWebUnit(
      new ClickURLLinkUnit("GoToListUserTab", "Click on the users tab").
      setURL("op=selectNode&objectId=usernode").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIListUser&op=viewUserInfo"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserInfo", "Click the view user info icon").
      setURL("op=viewUserInfo&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitNewMembership", "Assign 'newmembership' to the user in portal group").
      setFormId("membershipForm").
      setField(op:"save",membershipName:"newmembership",groupId:"/portal").
      addValidator(new ExpectTextValidator("newmembership"))
    ) ;
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("Back")
    ) ;
    //Go back to memebership tab and delete the new created memebership, then check
    //the new created membershipt that assign to an user, it should be removed as well
    addWebUnit(
      new ClickURLLinkUnit("GoToMembershipsTab", "Click on the memberships tab").
      setURL("op=selectNode&objectId=UIMembershipNode").
      addValidator(new ExpectLinkWithTextValidator("Add"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteNewMembership", "remove 'newmembership'").
      setURL("op=deleteMembership&objectId=newmembership").
      addValidator(new ExpectLinkWithURLValidator("op=deleteMembership&objectId=newmembership", 0))
    ) ;
    // Go to user info page and make sure the membership that assigned to him is removed
    addWebUnit(
      new ClickURLLinkUnit("GoToListUserTab", "Click on the users tab").
      setURL("op=selectNode&objectId=usernode").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIListUser&op=viewUserInfo"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserInfo", "Click the view user info icon").
      setURL("op=viewUserInfo&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectNoTextValidator("newmembership"))
    ) ;
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("Back")
    ) ;
    //Delete exoplatform account
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for user to delete").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",((String)EXOPLATFORM_ACCOUNT).substring(0,((String)EXOPLATFORM_ACCOUNT).length()-1)+"*" ).
      setField("op","search").
      addValidator(new ExpectNoTextValidator(EXOPLATFORM_ACCOUNT))
    ) ;
  }
  
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }
}
